@@ -384,7 +384,7 @@ tp_group_get_local_pending_cb (DBusGProxy *proxy,
 	EmpathyTpGroup     *group = user_data;
 	EmpathyTpGroupPriv *priv = GET_PRIV (group);
 	GArray             *handles;
-	guint               i;
+	guint               i = 0;
 	
 	if (error) {
 		empathy_debug (DEBUG_DOMAIN, "Failed to get local pendings: %s",
@@ -394,6 +394,7 @@ tp_group_get_local_pending_cb (DBusGProxy *proxy,
 	}
 
 	handles = g_array_sized_new (FALSE, FALSE, sizeof (guint), 1);
+	g_array_append_val (handles, i);
 	for (i = 0; array->len > i; i++) {
 		GValueArray        *pending_struct;
 		const gchar        *message;
@@ -407,7 +408,7 @@ tp_group_get_local_pending_cb (DBusGProxy *proxy,
 		reason = g_value_get_uint (g_value_array_get_nth (pending_struct, 2));
 		message = g_value_get_string (g_value_array_get_nth (pending_struct, 3));
 
-		g_array_insert_val (handles, 0, member_handle);
+		g_array_index (handles, guint, 0) = member_handle;
 		tp_group_members_changed_cb (priv->group_iface,
 					     message,      /* message */
 					     NULL,         /* added */
@@ -561,8 +562,8 @@ empathy_tp_group_new (McAccount *account,
 	DBusGProxy         *group_iface;
 	GError             *error = NULL;
 
-	g_return_val_if_fail (TELEPATHY_IS_CHAN (tp_chan), NULL);
 	g_return_val_if_fail (MC_IS_ACCOUNT (account), NULL);
+	g_return_val_if_fail (TELEPATHY_IS_CHAN (tp_chan), NULL);
 
 	group_iface = tp_chan_get_interface (tp_chan,
 					     TP_IFACE_QUARK_CHANNEL_INTERFACE_GROUP);
@@ -619,10 +620,12 @@ tp_group_async_cb (DBusGProxy *proxy, GError *error, gpointer user_data)
 }
 
 void
-empathy_tp_group_close  (EmpathyTpGroup *group)
+empathy_tp_group_close (EmpathyTpGroup *group)
 {
 	EmpathyTpGroupPriv *priv = GET_PRIV (group);
 
+	g_return_if_fail (EMPATHY_IS_TP_GROUP (group));
+
 	tp_chan_close_async (DBUS_G_PROXY (priv->tp_chan),
 			     tp_group_async_cb,
 			     "Failed to close");
@@ -678,6 +681,9 @@ empathy_tp_group_add_member (EmpathyTpGroup *group,
 	GArray             *handles;
 	guint               handle;
 
+	g_return_if_fail (EMPATHY_IS_TP_GROUP (group));
+	g_return_if_fail (EMPATHY_IS_CONTACT (contact));
+
 	handle = empathy_contact_get_handle (contact);
 	handles = g_array_new (FALSE, FALSE, sizeof (guint));
 	g_array_append_val (handles, handle);
@@ -721,6 +727,9 @@ empathy_tp_group_remove_member (EmpathyTpGroup *group,
 	GArray             *handles;
 	guint               handle;
 
+	g_return_if_fail (EMPATHY_IS_TP_GROUP (group));
+	g_return_if_fail (EMPATHY_IS_CONTACT (contact));
+
 	handle = empathy_contact_get_handle (contact);
 	handles = g_array_new (FALSE, FALSE, sizeof (guint));
 	g_array_append_val (handles, handle);
@@ -739,6 +748,8 @@ empathy_tp_group_get_members (EmpathyTpGroup *group)
 {
 	EmpathyTpGroupPriv *priv = GET_PRIV (group);
 
+	g_return_val_if_fail (EMPATHY_IS_TP_GROUP (group), NULL);
+
 	g_list_foreach (priv->members, (GFunc) g_object_ref, NULL);
 
 	return g_list_copy (priv->members);
@@ -750,6 +761,8 @@ empathy_tp_group_get_local_pendings (EmpathyTpGroup *group)
 	EmpathyTpGroupPriv *priv = GET_PRIV (group);
 	GList              *pendings = NULL, *l;
 
+	g_return_val_if_fail (EMPATHY_IS_TP_GROUP (group), NULL);
+
 	for (l = priv->local_pendings; l; l = l->next) {
 		EmpathyPendingInfo *info;
 		EmpathyPendingInfo *new_info;
@@ -769,6 +782,8 @@ empathy_tp_group_get_remote_pendings (EmpathyTpGroup *group)
 {
 	EmpathyTpGroupPriv *priv = GET_PRIV (group);
 
+	g_return_val_if_fail (EMPATHY_IS_TP_GROUP (group), NULL);
+
 	g_list_foreach (priv->remote_pendings, (GFunc) g_object_ref, NULL);
 
 	return g_list_copy (priv->remote_pendings);
@@ -819,6 +834,9 @@ empathy_tp_group_is_member (EmpathyTpGroup *group,
 {
 	EmpathyTpGroupPriv *priv = GET_PRIV (group);
 
+	g_return_val_if_fail (EMPATHY_IS_TP_GROUP (group), FALSE);
+	g_return_val_if_fail (EMPATHY_IS_CONTACT (contact), FALSE);
+
 	return g_list_find (priv->members, contact) != NULL;
 }
 
